/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#ifndef CLASS_HEURISTICINTERFACE_HEADER
#define CLASS_HEURISTICINTERFACE_HEADER

#include "../../aiconfig.h"
#include "../../../basistypes.h"

class Game;
class Hand;

namespace FORK_0_5_4 {

/// reine Virtuelle Klasse die fr Heuristics implementiert werden muss
class HeuristicInterface {
  public:	
    HeuristicInterface() { }
    virtual ~HeuristicInterface() { }

    virtual Team teamofplayer( Player const& player ) const = NULL;

    virtual Hand const& hand() const = NULL;
    virtual Team team() const = NULL;
    virtual unsigned no() const = NULL;
    virtual Game const& game() const = NULL;

    virtual bool value(const Aiconfig::TypeBool& type) const = NULL;
    virtual unsigned value(const Aiconfig::TypeUnsigned& type) const = NULL;
    virtual Card value(const Aiconfig::TypeCard& type) const = NULL;

    virtual unsigned playedcard(Card const& c) const = NULL;

    virtual bool colorjabbed(const Card::Color c) const = NULL;

    virtual unsigned color_runs(const Card::Color c) const = NULL;

    virtual bool jabbedbyownteam(const Card::Color c) const = NULL;
    virtual bool jabbedbyotherteam(const Card::Color c) const = NULL;

    virtual bool next_low() const = NULL;

    virtual Card trump_card_limit() const = NULL;
    virtual Card lowest_trump_card_limit() const = NULL;

    virtual Team const& teaminfo(Player const& player) const = NULL; 
}; // class HeuristicInterface

} // namespace FORK_0_5_4

#endif // #ifndef CLASS_HEURISTICINTERFACE_HEADER
